@@ -51,6 +51,12 @@ class PluginTest(testtools.TestCase):
             subnets=self.subnets,
             vlan=99)
 
+        self.network_ovs_trunk = objects.network.Network(
+            id='437c6db5-4e6f-4b43-b64b-ed6a11ee5ba7',
+            bridge='%s01' % constants.TRUNK_BR_PREFIX,
+            subnets=self.subnets,
+            vlan=99)
+
         self.network_ovs_mtu = objects.network.Network(
             id='437c6db5-4e6f-4b43-b64b-ed6a11ee5ba7',
             bridge='br0',
@@ -106,6 +112,14 @@ class PluginTest(testtools.TestCase):
             mode='client',
             port_profile=self.profile_ovs)
 
+        self.vif_vhostuser_trunk = objects.vif.VIFVHostUser(
+            id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
+            address='ca:fe:de:ad:be:ef',
+            network=self.network_ovs_trunk,
+            path='/var/run/openvswitch/vhub679325f-ca',
+            mode='client',
+            port_profile=self.profile_ovs)
+
         self.vif_vhostuser_client = objects.vif.VIFVHostUser(
             id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
             address='ca:fe:de:ad:be:ef',
@@ -173,8 +187,28 @@ class PluginTest(testtools.TestCase):
             interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'create_ovs_vif_port')
-    def test_create_vif_port_isolate(self, mock_create_ovs_vif_port):
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'port_exists')
+    def test_create_vif_port_isolate_port_no_isolate_vif_no_port(
+            self, mock_port_exists, mock_create_ovs_vif_port):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        mock_port_exists.return_value = False
+        with mock.patch.object(plugin.config, 'isolate_vif', False):
+            plugin._create_vif_port(
+                self.vif_ovs, mock.sentinel.vif_name, self.instance,
+                interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
+            mock_create_ovs_vif_port.assert_called_once_with(
+                self.vif_ovs.network.bridge, mock.sentinel.vif_name,
+                self.vif_ovs.port_profile.interface_id,
+                self.vif_ovs.address, self.instance.uuid,
+                mtu=plugin.config.network_device_mtu,
+                interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
+
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'create_ovs_vif_port')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'port_exists')
+    def test_create_vif_port_isolate_port_isolate_vif_no_port(
+            self, mock_port_exists, mock_create_ovs_vif_port):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        mock_port_exists.return_value = False
         with mock.patch.object(plugin.config, 'isolate_vif', True):
             plugin._create_vif_port(
                 self.vif_ovs, mock.sentinel.vif_name, self.instance,
@@ -187,20 +221,33 @@ class PluginTest(testtools.TestCase):
                 interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE,
                 tag=constants.DEAD_VLAN)
 
-    @mock.patch.object(ovs, 'sys')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'create_ovs_vif_port')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'port_exists')
+    def test_create_vif_port_isolate_port_isolate_vif_port_exists(
+            self, mock_port_exists, mock_create_ovs_vif_port):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        mock_port_exists.return_value = True
+        with mock.patch.object(plugin.config, 'isolate_vif', True):
+            plugin._create_vif_port(
+                self.vif_ovs, mock.sentinel.vif_name, self.instance,
+                interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
+            mock_create_ovs_vif_port.assert_called_once_with(
+                self.vif_ovs.network.bridge, mock.sentinel.vif_name,
+                self.vif_ovs.port_profile.interface_id,
+                self.vif_ovs.address, self.instance.uuid,
+                mtu=plugin.config.network_device_mtu,
+                interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
+
     @mock.patch.object(ovs.OvsPlugin, '_plug_vif_generic')
-    def test_plug_ovs_port_bridge_false(self, plug_vif_generic, mock_sys):
-        mock_sys.platform = 'linux'
+    def test_plug_ovs_port_bridge_false(self, plug_vif_generic):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         with mock.patch.object(plugin.config, 'per_port_bridge', False):
             plugin.plug(self.vif_ovs, self.instance)
             plug_vif_generic.assert_called_once_with(
                 self.vif_ovs, self.instance)
 
-    @mock.patch.object(ovs, 'sys')
     @mock.patch.object(ovs.OvsPlugin, '_plug_port_bridge')
-    def test_plug_ovs_port_bridge_true(self, plug_vif, mock_sys):
-        mock_sys.platform = 'linux'
+    def test_plug_ovs_port_bridge_true(self, plug_vif):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         with mock.patch.object(plugin.config, 'per_port_bridge', True):
             plugin.plug(self.vif_ovs, self.instance)
@@ -225,8 +272,7 @@ class PluginTest(testtools.TestCase):
     @mock.patch.object(linux_net, 'create_veth_pair')
     @mock.patch.object(ip_lib, 'exists')
     @mock.patch.object(linux_net, 'ensure_bridge')
-    @mock.patch.object(ovs, 'sys')
-    def test_plug_ovs_bridge(self, mock_sys, ensure_bridge, device_exists,
+    def test_plug_ovs_bridge(self, ensure_bridge, device_exists,
                              create_veth_pair, update_veth_pair,
                              add_bridge_port, _create_vif_port,
                              _update_vif_port, ensure_ovs_bridge,
@@ -256,7 +302,6 @@ class PluginTest(testtools.TestCase):
         # plugging new devices should result in devices being created
 
         device_exists.return_value = False
-        mock_sys.platform = 'linux'
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(self.vif_ovs_hybrid, self.instance)
         ensure_bridge.assert_has_calls(calls['ensure_bridge'])
@@ -282,85 +327,50 @@ class PluginTest(testtools.TestCase):
         update_veth_pair.assert_has_calls(calls['update_veth_pair'])
         _update_vif_port.assert_has_calls(calls['_update_vif_port'])
 
-    @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
-    @mock.patch.object(ovs.OvsPlugin, '_create_vif_port')
-    @mock.patch.object(ip_lib, 'exists', return_value=False)
-    @mock.patch.object(ovs, 'sys')
-    def _check_plug_ovs_windows(self, vif, mock_sys, mock_exists,
-                                _create_vif_port, ensure_ovs_bridge):
-        dp_type = ovs.OvsPlugin._get_vif_datapath_type(vif)
-        calls = {
-            'exists': [mock.call(vif.id)],
-            '_create_vif_port': [mock.call(vif, vif.id, self.instance)],
-            'ensure_ovs_bridge': [mock.call('br0', dp_type)]
-        }
-
-        mock_sys.platform = constants.PLATFORM_WIN32
-        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
-        plugin.plug(vif, self.instance)
-        mock_exists.assert_has_calls(calls['exists'])
-        _create_vif_port.assert_has_calls(calls['_create_vif_port'])
-        ensure_ovs_bridge.assert_has_calls(calls['ensure_ovs_bridge'])
-
-    def test_plug_ovs_windows(self):
-        self._check_plug_ovs_windows(self.vif_ovs)
-
-    def test_plug_ovs_bridge_windows(self):
-        self._check_plug_ovs_windows(self.vif_ovs_hybrid)
-
-    @mock.patch.object(ovs, 'sys')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
-    def test_unplug_ovs_port_bridge_false(self, unplug, mock_sys):
-        mock_sys.platform = 'linux'
+    def test_unplug_ovs_port_bridge_false(self, unplug,
+                                          delete_ovs_bridge):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         with mock.patch.object(plugin.config, 'per_port_bridge', False):
             plugin.unplug(self.vif_ovs, self.instance)
             unplug.assert_called_once_with(self.vif_ovs, self.instance)
+        delete_ovs_bridge.assert_not_called()
 
-    @mock.patch.object(ovs, 'sys')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_unplug_port_bridge')
-    def test_unplug_ovs_port_bridge_true(self, unplug, mock_sys):
-        mock_sys.platform = 'linux'
+    def test_unplug_ovs_port_bridge_true(self, unplug,
+                                         delete_ovs_bridge):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         with mock.patch.object(plugin.config, 'per_port_bridge', True):
             plugin.unplug(self.vif_ovs, self.instance)
             unplug.assert_called_once_with(self.vif_ovs, self.instance)
+        delete_ovs_bridge.assert_not_called()
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
-    def test_unplug_vif_generic(self, delete_port):
+    def test_unplug_vif_generic(self, delete_port, delete_ovs_bridge):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin._unplug_vif_generic(self.vif_ovs, self.instance)
         delete_port.assert_called_once()
+        delete_ovs_bridge.assert_not_called()
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
     @mock.patch.object(linux_net, 'delete_bridge')
-    @mock.patch.object(ovs, 'sys')
-    def test_unplug_ovs_bridge(self, mock_sys, delete_bridge,
-                               delete_ovs_vif_port):
+    def test_unplug_ovs_bridge(self, delete_bridge,
+                               delete_ovs_vif_port, delete_ovs_bridge):
         calls = {
             'delete_bridge': [mock.call('qbrvif-xxx-yyy', 'qvbb679325f-ca')],
-            'delete_ovs_vif_port': [mock.call('br0', 'qvob679325f-ca')]
+            'delete_ovs_vif_port': [mock.call(
+                'br0', 'qvob679325f-ca', qos_type='linux-noop'
+            )]
         }
-        mock_sys.platform = 'linux'
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(self.vif_ovs_hybrid, self.instance)
         delete_bridge.assert_has_calls(calls['delete_bridge'])
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
-
-    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
-    @mock.patch.object(ovs, 'sys')
-    def _check_unplug_ovs_windows(self, vif, mock_sys, delete_ovs_vif_port):
-        mock_sys.platform = constants.PLATFORM_WIN32
-        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
-        plugin.unplug(vif, self.instance)
-        delete_ovs_vif_port.assert_called_once_with('br0', vif.id,
-                                                    delete_netdev=False)
-
-    def test_unplug_ovs_windows(self):
-        self._check_unplug_ovs_windows(self.vif_ovs)
-
-    def test_unplug_ovs_bridge_windows(self):
-        self._check_unplug_ovs_windows(self.vif_ovs_hybrid)
+        delete_ovs_bridge.assert_not_called()
 
     @mock.patch.object(ovs.OvsPlugin, '_create_vif_port')
     def test_plug_ovs_vhostuser(self, _create_vif_port):
@@ -394,14 +404,30 @@ class PluginTest(testtools.TestCase):
         plugin.plug(self.vif_vhostuser_client, self.instance)
         create_ovs_vif_port.assert_has_calls(calls)
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
-    def test_unplug_ovs_vhostuser(self, delete_ovs_vif_port):
+    def test_unplug_ovs_vhostuser(self, delete_ovs_vif_port,
+                                  delete_ovs_bridge):
         calls = {
             'delete_ovs_vif_port': [mock.call('br0', 'vhub679325f-ca')]
         }
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(self.vif_vhostuser, self.instance)
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
+        delete_ovs_bridge.assert_not_called()
+
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
+    def test_unplug_ovs_vhostuser_trunk(self, delete_ovs_vif_port,
+                                        delete_ovs_bridge):
+        bridge_name = '%s01' % constants.TRUNK_BR_PREFIX
+        calls = {
+            'delete_ovs_vif_port': [mock.call(bridge_name, 'vhub679325f-ca')]
+        }
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin.unplug(self.vif_vhostuser_trunk, self.instance)
+        delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
+        delete_ovs_bridge.assert_called_once_with(bridge_name)
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
     @mock.patch.object(linux_net, 'get_ifname_by_pci_address')
@@ -446,6 +472,7 @@ class PluginTest(testtools.TestCase):
         set_interface_state.assert_has_calls(calls['set_interface_state'])
         _create_vif_port.assert_has_calls(calls['_create_vif_port'])
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(linux_net, 'get_ifname_by_pci_address')
     @mock.patch.object(linux_net, 'get_vf_num_by_pci_address')
     @mock.patch.object(linux_net, 'get_representor_port')
@@ -455,7 +482,8 @@ class PluginTest(testtools.TestCase):
                                      set_interface_state,
                                      get_representor_port,
                                      get_vf_num_by_pci_address,
-                                     get_ifname_by_pci_address):
+                                     get_ifname_by_pci_address,
+                                     delete_ovs_bridge):
         calls = {
 
             'get_ifname_by_pci_address': [mock.call('0002:24:12.3',
@@ -464,8 +492,12 @@ class PluginTest(testtools.TestCase):
             'get_vf_num_by_pci_address': [mock.call('0002:24:12.3')],
             'get_representor_port': [mock.call('eth0', '2')],
             'set_interface_state': [mock.call('eth0_2', 'down')],
-            'delete_ovs_vif_port': [mock.call('br0', 'eth0_2',
-                                              delete_netdev=False)]
+            'delete_ovs_vif_port': [
+                mock.call(
+                    'br0', 'eth0_2', delete_netdev=False,
+                    qos_type='linux-noop'
+                )
+            ]
         }
 
         get_ifname_by_pci_address.return_value = 'eth0'
@@ -481,6 +513,7 @@ class PluginTest(testtools.TestCase):
             calls['get_representor_port'])
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
         set_interface_state.assert_has_calls(calls['set_interface_state'])
+        delete_ovs_bridge.assert_not_called()
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, "_create_vif_port")
@@ -491,12 +524,14 @@ class PluginTest(testtools.TestCase):
             ensure_bridge.assert_called_once()
             create_port.assert_called_once()
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
-    def test_unplug_vif_ovs_smart_nic(self, delete_port):
+    def test_unplug_vif_ovs_smart_nic(self, delete_port, delete_ovs_bridge):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         with mock.patch.object(plugin.config, 'per_port_bridge', False):
             plugin.unplug(self.vif_ovs_smart_nic, self.instance)
             delete_port.assert_called_once()
+        delete_ovs_bridge.assert_not_called()
 
     @mock.patch.object(linux_net, 'get_dpdk_representor_port_name')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
@@ -542,22 +577,30 @@ class PluginTest(testtools.TestCase):
         _create_vif_port.assert_has_calls(
             calls['_create_vif_port'])
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(linux_net, 'get_dpdk_representor_port_name')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
     def test_unplug_ovs_vf_dpdk(self, delete_ovs_vif_port,
-                                get_dpdk_representor_port_name):
+                                get_dpdk_representor_port_name,
+                                delete_ovs_bridge):
         devname = 'vfrb679325f-ca'
         get_dpdk_representor_port_name.return_value = devname
         calls = {
             'get_dpdk_representor_port_name': [mock.call(
                 self.vif_ovs_vf_dpdk.id)],
-            'delete_ovs_vif_port': [mock.call('br0', devname,
-                                              delete_netdev=False)]}
+            'delete_ovs_vif_port': [
+                mock.call(
+                    'br0', devname, delete_netdev=False,
+                    qos_type=None
+                )
+            ]
+        }
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(self.vif_ovs_vf_dpdk, self.instance)
         get_dpdk_representor_port_name.assert_has_calls(
             calls['get_dpdk_representor_port_name'])
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
+        delete_ovs_bridge.assert_not_called()
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'create_patch_port_pair')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
@@ -584,7 +627,21 @@ class PluginTest(testtools.TestCase):
             mock.call('br0', 'ibpb679325f-ca89-4ee0-a8be-6db1409b69ea'),
             mock.call(
                 'pbb679325f-ca8', 'pbpb679325f-ca89-4ee0-a8be-6db1409b69ea'),
-            mock.call('pbb679325f-ca8', 'tap-xxx-yyy-zzz')
+            mock.call('pbb679325f-ca8', 'tap-xxx-yyy-zzz', qos_type=None)
         ]
         delete_ovs_vif_port.assert_has_calls(calls)
         delete_ovs_bridge.assert_called_once_with('pbb679325f-ca8')
+
+    @mock.patch.object(ip_lib, 'exists', return_value=True)
+    @mock.patch.object(ovs.OvsPlugin, '_unplug_bridge')
+    def test_unplug_hybrid_bridge(self, m_unplug_bridge, m_ip_lib_exists):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin.unplug(self.vif_ovs, self.instance)
+        m_unplug_bridge.assert_called_once()
+
+    @mock.patch.object(ip_lib, 'exists', return_value=False)
+    @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
+    def test_unplug_ovs(self, m_unplug_generic, m_ip_lib_exists):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin.unplug(self.vif_ovs, self.instance)
+        m_unplug_generic.assert_called_once()
